# $Id: Crypto.pm 30791 2010-04-22 13:53:18Z wsl $
# $URL: https://infix.uvt.nl/its-id/trunk/sources/loandesk/lib/UvT/Loandesk/Crypto.pm $

use utf8;
use strict;
use warnings FATAL => 'all';

package UvT::Loandesk::Crypto;

use MIME::Base64;
use Digest::SHA qw(sha256);
use Crypt::OpenSSL::Random;
use Time::HiRes qw(time);

use Exporter 'import';

our @EXPORT = qw(loandesk_random_bytes loandesk_random_hex loandesk_create_token loandesk_check_token);

sub loandesk_random_bytes {
	die "0001 Insufficient entropy\n"
		unless Crypt::OpenSSL::Random::random_status;
	return Crypt::OpenSSL::Random::random_bytes(@_);
}

sub loandesk_random_hex {
	my $hex = unpack('H*', loandesk_random_bytes($@));
	$hex =~ tr/a-z/A-Z/;
	return $hex;
}

sub loandesk_create_token {
	my ($secret, @keys) = @_;

	foreach(@keys) {
		utf8::encode($_);
		die "Bad characters in request\n"
			unless index($_, $;) < 0;
	}

	my $time = pack('Q>', int(time * 1e6));
	my $salt = loandesk_random_bytes(16);

	my $token = $time . $salt . join($;, @keys);
	my $hash = sha256($secret, $;, $token);
	my $encoded = encode_base64($hash.$token, '');
	return wantarray ? ($time, $salt, $encoded) : $encoded;
}

sub loandesk_check_token {
	my ($secret, $token) = @_;

	die "Invalid token\n"
		unless defined $token;

	die "Invalid token\n"
		unless length($token) % 4 == 0;

	die "Invalid token\n"
		unless $token =~ m|^[a-zA-Z0-9+/]+={0,2}$|;

	$token = decode_base64($token);

	my $hash = substr($token, 0, 32, '');
	die "Invalid token\n"
		unless length($hash) == 32;

	die "Invalid token\n"
		unless sha256($secret, $;, $token) eq $hash;

	my $time = unpack('Q>', substr($token, 0, 8, '')) / 1e6;

	my $salt = substr($token, 0, 16, '');

	return ($time, $salt, split($;, $token));
}
